// *********************************************************************
// *                                                                   *
// * (c) 1997-2004 Andreas Ziermann LOET.DE                            *
// *                                                                   *
// * These files are for reference only. You may not use these files   *
// * for commercial purpose use without permit.                        *
// *                                                                   *
// * b@loet.de                                                         *
// *                                                                   *
// * 1.00 2004-07-04                                                   *
// *      first released                                               *
// *                                                                   *
// *********************************************************************

#ifndef __AZFILE_H__
#define __AZFILE_H__

class AZFile
{
protected:
  AZString filename;
  int bad;
  FILE *fh; 
  AZFile(const char *fname,const char *mode);
  virtual ~AZFile();
public:
  int Seek(long offset,int whence) { assert(fh); return fseek(fh,offset,whence); }
  // ist das file noch in ordnung?
  // nach dem Oeffnen z.b. steht in bad der errorcode
  int IsBad() const { return bad; }
  int Eof() const { return feof(fh); }
  // obsolete funktion
  // wenn ein file geoeffnet werden konnte -> ret=false;
  // wenn *fs==0 ist dann true;
  int operator !() const;
  const char *Filename() const { return filename.Text(); }
};

class AZWriteFile : public AZFile
{
protected:

  // Write() returns the number of bytes written
  void Write(byte c);
  void Write(word w);  // files always written in intel-byte-order
                       // sometimes this 2 bytes will be swapped
  void Write(dword d); // ditto 4 bytes swapping
public:
  AZWriteFile & operator<< (byte c) { Write(c); return *this; }
  AZWriteFile & operator<< (word w) { Write(w); return *this; }
  AZWriteFile & operator<< (dword d){ Write(d); return *this; }

  AZWriteFile & operator<< (char c) { Write((byte)c); return *this; }
  AZWriteFile & operator<< (short w){ Write((word)w); return *this; }
  AZWriteFile & operator<< (int d)  { Write((dword)d); return *this; }
  // AZWriteFile & operator<< (const char *str) { Write(str,strlen(str)+1); return *this; }
  
  void Write(const void *buf,size_t len); // no swapping!!!

  // for mode details - refer fopen
  AZWriteFile(const char *name,const char *mode=NULL);
  ~AZWriteFile();
};

class AZReadFile : public AZFile
{
  long fileLen;  
  long bytesToRead;

protected:

  // Read() returns the number of bytes written
  void Read(byte &c);
  void Read(word &w);  // files always written in intel-byte-order
                       // sometimes this 2 bytes will be swapped
  void Read(dword &d); // ditto 4 bytes swapping
public:
  AZReadFile & operator>> (byte &c) { Read(c); return *this; }
  AZReadFile & operator>> (word &w) { Read(w); return *this; }
  AZReadFile & operator>> (dword &d){ Read(d); return *this; }

  AZReadFile & operator>> (char &c) { Read((byte&)c); return *this; }
  AZReadFile & operator>> (short &w){ Read((word&)w);  return *this; }
  AZReadFile & operator>> (int &d)  { Read((dword&)d); return *this; }
  void Read(void *buf,size_t len); // no swapping!!!
  void ReadLine(AZString &);

  AZReadFile(const char *name);
};

class AZFileCreateMsg
{
public:
  virtual void MsgFileCreate() {};  
  AZFileCreateMsg();
  virtual ~AZFileCreateMsg();
};

class FileCreateMsgList :  public  AZList <AZFileCreateMsg> 
{
public:
  void MsgFileCreate();
};

// directory enlesen
typedef struct
{
  AZString name;
  bool     isDir;
} DirInfo;

typedef AZDList <DirInfo> AZDirList;

// ret = anzahl der eintraege
// hier wird alphabetisch sortiert
// (sollte eigentlich - die funktion gibt es nicht ueberall also ist sie hier) 
int GetDirSort(const char *startdir, AZDirList &list);

#endif

