// *********************************************************************
// *                                                                   *
// * (c) 1997-2004 Andreas Ziermann LOET.DE                            *
// *                                                                   *
// * These files are for reference only. You may not use these files   *
// * for commercial purpose use without permit.                        *
// *                                                                   *
// * b@loet.de                                                         *
// *                                                                   *
// * 1.00 2004-07-04                                                   *
// *      first released                                               *
// *                                                                   *
// *********************************************************************

#ifndef __AZLIST_H__
#define __AZLIST_H__

class AZListNode
{
  friend class AZVoidList;
  friend class AZVoidListIter;

  AZListNode *next;
  AZListNode *prev;
  void *data;
public:
  AZListNode(AZListNode *_next,AZListNode *_prev,void *_data)
  :next(_next)
  ,prev(_prev)
  ,data(_data)
  {}
        void       *Data()       { return data; }
  const void       *Data() const { return data; }
        AZListNode *Next()       { return next; }
  const AZListNode *Next() const { return next; }
};

class AZVoidList
{
  friend class AZVoidListIter;
protected:
  AZListNode *first,*last;
public:
  AZVoidList():first(0),last(0) {}
  ~AZVoidList();

  void AddTail(void *_data);
        AZListNode *First() { return first; }
  const AZListNode *First() const { return first; }
        AZListNode *Find(const void *data);
  const AZListNode *Find(const void *data) const;
        void       *Remove(const void *_arg);
  void  DeleteAll();
};

class AZVoidListIter
{
protected:
  AZVoidList &liste;
  AZListNode *node;
public:
  AZVoidListIter(AZVoidList &l):liste(l),node(l.First()) {}
  void *operator ++(int){ AZListNode *n=node; node=node->Next(); return n; }
  void *operator ++()   { return node=node->Next(); }
  void *operator () ()  { return node->Data(); }
  operator bool() const { return node?true:false; }
  // zerstoert den listknoten und gibt die daten zurueck;
  void *Remove();
};

template <class DATA> class AZList : public AZVoidList
{
public:
  void AddTail(DATA *_data) { AZVoidList::AddTail((void*)_data); }
  AZListNode *Find(const DATA *_arg)
  {
    return AZVoidList::Find(_arg);
  }
  const AZListNode *Find(const DATA *_arg) const
  {
    return AZVoidList::Find(_arg);
  }
  DATA *Remove(const DATA *_arg)
  {
    return (DATA*)AZVoidList::Remove(_arg);
  }
  void DeleteAll()
  {
    AZListNode *f=AZVoidList::First();
    while(f)
    {
      assert(f->Data());
      delete (DATA *)f->Data();
      AZListNode *toDel=f;
      f=f->Next();
      delete toDel;
    }
    first=0;last=0;
  }
};

template <class DATA> class AZListIter: public AZVoidListIter
{
public:
  //AZListIter(const AZVoidList &l):AZVoidListIter((AZVoidList &)l) {}
  AZListIter(AZVoidList &l):AZVoidListIter(l) {}
  DATA & operator ++(int) { return *(DATA *)AZVoidListIter::operator ++ (0); }
  DATA & operator ++()    { return *(DATA *)AZVoidListIter::operator ++ ();  }
  operator DATA & ()      { return *(DATA *)node->Data();}
  DATA & operator ()()    { return *(DATA *)node->Data();}
};


template <class DATA> class AZDList : public AZList <DATA>
{
public:
  ~AZDList() { DeleteAll(); }
};

#define FORALL(args,AZArgument) for(AZListIter <AZArgument> it(args);(bool)it;it++)

#endif
