// *********************************************************************
// *                                                                   *
// * (c) 1997-2004 Andreas Ziermann LOET.DE                            *
// *                                                                   *
// * These files are for reference only. You may not use these files   *
// * for commercial use without permit.                                *
// *                                                                   *
// * b@loet.de                                                         *
// *                                                                   *
// * 1.00 2004-07-04                                                   *
// *      first released                                               *
// *                                                                   *
// *********************************************************************
void heap_check();
void heap_free_count();

// groesse des bootmanagers in sektoren
#define MAX_SECTORLEN 8

#ifdef __WATCOM__
  #define r_ax w.ax
  #define r_bx w.bx
  #define r_cx w.cx
  #define r_dx w.dx
  #define r_si w.si
  #define r_ds w.ds
  #define r_es w.es
  #define r_flags w.flags

  #define FA_LABEL 0x08 
#endif

typedef word zylsec; // 8-bit zylinder / 2-bitzylinder+6 bit sektor

struct LbaREF
{
  byte  hdnr;
  dword lbaADR;
};

struct PartitionEntry
{
  byte   active; //80h aktiv; 00
  byte   start_head;
  zylsec start_zyl;
  byte   part_id;
  byte   end_head;
  zylsec end_zyl;
  dword  rel_start;
  dword  sec_count;
};                     // 16 byte

struct PartitionTableNeu
{
  byte code  [0x1b0];
  byte fill  [0xe];
  PartitionEntry entry[4];
  word signatur; //aa55
};

class PartitionTableHeap
{
  PartitionTableNeu *p;
//  PartitionTableNeu p;
public:
  PartitionTableHeap() { p=new PartitionTableNeu; }
  ~PartitionTableHeap() { delete p; }

  // PartitionTableNeu & PE() { return p; }
  PartitionTableNeu & operator()() { return *p; }
};

struct BootSectorFAT32
{
  byte  jmp[3];
  char  oemid[8];
  byte  ___[60];
  char vollabel[11];
  char fsid[8];
  byte code [512-92];
  word signatur;
};

struct BootSectorOS2
{
  byte  jmp[3];
  char  oemid[8];
  byte  ___[32];
  char  vollabel[11]; //hoffentlich stimmts
  char  fsid[8];
  byte  code[512-64];
  word  signatur;
};

struct BootSectorNTFS
{
  byte jmp[3];
  char oemid[8];
  byte code [512-13];
  word signatur;
};

struct BootSectorFAT
{
  byte jmp[3];
  char oemid[8];
  word  bytes_per_sector;    // 11
  byte  sectors_per_cluster; // 13
  word  sectors_reserved;    // 14 normally 1 - bootsector
  byte  fat_copies;          // 16 # fat - copies
  word  main_dir_entries;    // 17
  word  sectors_per_disk;    // 19 ?? cluster??
  byte  media_descriptor;    // 21
  word  sectors_per_fat;     // 22
  word  sectors_per_track;   // 24
  word  head_count;          // 26
  dword hidden_sectors;      // 28
  dword sectors_at_all;      // 32
  word  hdnr;                // 36
  byte  enhanced_start_sig;  // 38
  dword serial_number;       // 39
  char vollabel[11];
  char fsid[8];
  byte code [512-64];
  word signatur;
};

struct BootSectorLinux
{
  byte jmp[510];
  word signatur;
};

struct BootSector
{
  byte jmp[510];
  word signatur;
};

struct DirEntrie
{
  char  name[8];
  char  ext[3];
  byte  attribute;
  char  reserved[10];
  word  uhrzeit; // der letzten aenderung
  word  datum;   // der letzten aenderung
  word  cluster_ref; // 1.cluster der datei;
  dword len; // in bytes
};

// aufbau des startup codes
struct MBR_Mr_Boot
{                       // eb 0d
  byte jmp[2];          // jmp nach code muesste also
  char mrboot[6];       // 'MrBoot'
  char boot_version[5]; // z.B. '4.08d'
  byte boot_disk;       // hd# des bootmanagers z.B. hd0=0x80 oder fd0=0
  byte boot_seccount;   // laenge des bootmanagers in sectoren

  byte code  [0x1a1];
  // offset 1b0;
  byte fill  [0xe];     // besondere verwendung in anderes OS - passwort etc,
                        // deswegen darf der startupcode nur bis 0x1b0 reichen
  PartitionEntry entry[4];
  word signatur; //aa55
};

struct Par_Mr_Boot
{
  byte jmp[3];
  word par_entry_size; // sonne art versionsnummer laenge aller eintraege
  word par_partition_entry_len; // laenge eines partitionseintrages
  word par_partition_entry_count; // anzahl der partitionseintraege
  word par_partition_starts_offset; // offset zur tabelle
  byte par_test_mode; // muesste auf immer 0 sein - das er ja nicht im
                      // testmode auf arbeitet, wenn er auf disk/platte
                      // installiert worden ist
                      // wartet bevor der bootmanager anspringt
  //CFG_BEEP_BEIMSTART   EQU 1
  //CFG_NO_WRITE         EQU 2
  byte par_config; // Konfiguration des bootmanagers
  word par_startverzoegerung; // zeige biosmeldungen bevor MB anspringt
  word par_byte_len; //_main_boot_ends-_main_boot gesamtlaenge des BM
  byte par_cols[4];      // COL_NORMAL COL_IMPORTANT COL_MENU  COL_SEL
  // timeout in sekunden oder 0xffff fuer unbeschraenkt
  word par_wartezeit; // sollte zwischen 1..99 liegen
  // hier endet unser parameterblock
};

union Sector
{
  byte              code[512];
  BootSector        bootsector;
  BootSectorFAT     bootsectorfat;
  BootSectorFAT32   bootsectorfat32;
  BootSectorLinux   bootsectorlinux;
  BootSectorOS2     bootsectoros2;
  BootSectorNTFS    bootsectorNTFS;
  DirEntrie         dir_entries[16];
  MBR_Mr_Boot       mrboot;
  Par_Mr_Boot       bootmgr; // 1. sektor des BM
  PartitionTableNeu partition_table;
};

void     BIOSBeep                ();
unsigned BIOSHarddiskCount       ();
bool     BIOSGetPartitionTable   (unsigned char hdnr,PartitionTableNeu *t);
// follow extended partition
//bool     BIOSGetPartitionTableExt(unsigned char hdnr,const PartitionEntry &en,
//                                  PartitionTableNeu *t);
bool     BIOSGetPartitionTableExt(unsigned char hdnr,dword lbaADR,
                                  PartitionTableNeu *t);
// ist jetzt private
//bool     BIOSGetSector           (unsigned char hdnr,byte head, zylsec zyl,
//                                  Sector *sector);
bool     BIOSLoadSectorLBA(byte hdnr,dword lbaADR,Sector *b);

bool     BIOSWriteBootMgrToHead0 (int hdnr,const char far *buf,
                                  int byteLen);
bool     BIOSWriteMBR            (int hdnr,const Sector *buf);

enum MODE
{
  MODE_80x25_CO  = 3,
  MODE_HERC_MONO = 7
};

void     BIOSSetMode             (MODE mode);
void     BIOSRestoreMode         ();

// typ des bootsektors, daran kann man entscheiden, ob
//

enum PAR_TYPE
{
  UNKNOWN,
  UNKNOWNDOS,
  WIN9X,
  FAT32,
  DRDOS60,
  NWDOS7,
  OPENDOS7,
  MSDOS401,
  LINUX,
  OS2BOOTMAN,
  OS2HPFS,
  NTFS,
  IBMDOS,
  FLOPPYBOOTSEC,
};

struct PAR_TYPE_ENTRY
{
  PAR_TYPE type;
  byte     flags;
};

void BIOSSaveScreen();
void BIOSRestoreScreen();

class PartEntry;
void BIOSFat16_Fat12_FindVolLabel(AZString &volLabel,byte hdnr,
                           const BootSectorFAT &bs,dword lba_boot);

struct scr_char
{
  unsigned char zeichen;
  byte attribut;
};

#ifdef __WATCOM__
  #define CYAN 3
  #define YELLOW 14
  #define RED    4
  #define WHITE  15
  void textattr(int );
  void gotoxy(int ,int ) ;
  void window(int ,int ,int ,int ); 
  void clrscr(); 
  #define _NOCURSOR 0
  void _setcursortype(int); 
#endif
