// *********************************************************************
// *                                                                   *
// * (c) 1997-2004 Andreas Ziermann LOET.DE                            *
// *                                                                   *
// * These files are for reference only. You may not use these files   *
// * for commercial use without permit.                                *
// *                                                                   *
// * b@loet.de                                                         *
// *                                                                   *
// * 1.00 2004-07-04                                                   *
// *      first released                                               *
// *                                                                   *
// *********************************************************************
#define PAR_X 0
#define PAR_Y 9+2
#define PAR_W 78
#define PAR_H 12-2

// listbox parameter
#define LB_DEFAULT      0
#define LB_NOVSCROLLBAR 1

// definitionen fuer die ListItem id's
#define LISTITEM_PARTENTRY 0x100
// ... bis 0x1ff
#define LISTITEM_SELWINDOW 0x200
// ... bis 0x2ff
#define LISTITEM_PARTEDIT  0x300
// ... bis 0x3ff
#define LISTITEM_SELBOOT   0x400
// ... bis 0x4ff
#define LISTITEM_STARTVERZOEGERUNG 0x500
#define LISTITEM_WARTEZEIT         0x501
#define LISTITEM_UNBESCHRAENKT     0x502
#define LISTITEM_BEEPBEIMSTART     0x503
#define LISTITEM_INSTALLTOMBR      0x504
#define LISTITEM_INSTALLTODISK     0x505
#define LISTITEM_MONO              0x506

// argument "1" gibt den string an, in den die anzahl der
// sekunden stehen
#define INDEX_STARTVERZOEGERUNG 1
#define INDEX_WARTEZEIT         1
#define INDEX_UNBESCHRAENKT     1
#define INDEX_BEEPBEIMSTART     1
#define INDEX_MONO              1


#define INDEX_HIDING_MAP        9
#define INDEX_UNSICHTBAR        11

//#define USE_DBLLINE
//#define USE_HALFCHAR
#define USE_SGLLINE

#ifdef USE_DBLLINE

#define TOP_LEFT     ''
#define TOP          ''
#define TOP_RIGHT    ''
#define LEFT         ''
#define BOTTOM_LEFT  ''
#define BOTTOM       ''
#define BOTTOM_RIGHT ''
#define RIGHT        ''
#define SPACE        ' '
#define BR_RIGHT     ''
#define BR_BOTTOM    ''
#define BR_TOP       ''
#define BR_LEFT      ''



#endif

#ifdef USE_HALFCHAR


#define TOP_LEFT     ''
#define TOP          ''
#define TOP_RIGHT    ''
#define LEFT         ''
#define BOTTOM_LEFT  ''
#define BOTTOM       ''
#define BOTTOM_RIGHT ''
#define RIGHT        ''
#define SPACE        ' '
#define BR_RIGHT     ''
#define BR_BOTTOM    ''
#define BR_TOP       ''
#define BR_LEFT      ''

#endif

#ifdef USE_SGLLINE

#define TOP_LEFT     ''
#define TOP          ''
#define TOP_RIGHT    ''
#define LEFT         ''
#define BOTTOM_LEFT  ''
#define BOTTOM       ''
#define BOTTOM_RIGHT ''
#define RIGHT        ''
#define SPACE        ' '
#define BR_RIGHT     ''
#define BR_BOTTOM    ''
#define BR_TOP       ''
#define BR_LEFT      ''

#endif

#define COL_TITLE     cols[0]
#define COL_NORMAL    cols[1]
#define COL_SELECT    cols[2]
#define COL_KEY       cols[3]
#define COL_HEADLINE  cols[4]

// blinkend + schwarzer grund / normale schrift+unterstrichen
#define COL_CUR       cols[5]
#define COL_EDIT      cols[6]
#define COL_IMPORTANT cols[7]
#define COL_BW        cols[8]

// msgbox
#define COL_ALTERT    cols[9]

extern const byte *cols;

#define WIN_NO_FRAME 1

class Window
{

protected:
  int x,y,w,h;
  void gotoxy(int x,int y);

public:
  Window(int _x,int _y,int _w,int _h);
  virtual ~Window() {}
  virtual void Paint();
};

class CWindow
{

protected:
  int x,y,w,h;
  word flags;
  byte attr;

  word buffer_size;
  byte *buffer_attrib;
  byte *buffer_char;

  void gotoxy(int x,int y);

protected:
  void Clear(); //loescht das ganze gebiet mit der farbe von SetColor
  void SetColor(byte attr);
  void DrawText(int relx, int rely, const char *txt,byte col=0);

public:
  CWindow(int _x,int _y,int _w,int _h,word flags=0);
  virtual ~CWindow() {}
  virtual void Paint();

  word Execute(); // modales fenster
  void Center();
};

class ListItemBase
{
  AZString item;
  int      id; // suchen nach ids
public:
  ListItemBase(const char *txt,int _id)
    :item(txt),id(_id) {}
  const char *Text() const { return item.Text(); }
  virtual ~ListItemBase() {}
  int GetID() const { return id; }
};

class ListItem: public ListItemBase
{
  void     *data;
public:
  ListItem(const char *txt,void *_data,int id)
    :ListItemBase(txt,id),data(_data) {}
  void *Data() { return data; }
  const void *Data() const { return data; }
};

class PartEntry;
class EditList;

class ListItemPartEntry : public ListItemBase
{
  const PartEntry  & pe;
public:
  ListItemPartEntry(const char *txt,int id,
                    const PartEntry & _pe)
    :ListItemBase(txt,id),pe(_pe) {}
  const PartEntry & GetPartEntry() const { return pe; }
};

class ListItemEditList : public ListItemBase
{
  EditList         *edl;
public:
  ListItemEditList(EditList * _edl,int id)
    :ListItemBase("",id),edl(_edl) {}
  EditList *GetEditList()       { return edl; }
  void SetEditList(EditList *l) { edl=l; }
  const EditList & GetEditList()  const { return *edl; }
};

class ListItemSelWindow : public ListItemPartEntry
{
//  const PartEntry  *pe;
  EditList         *edl;
public:
  ListItemSelWindow(const char *txt,int id,
                    const PartEntry& pe,EditList *_edl)
    :ListItemPartEntry(txt,id,pe),edl(_edl) {}
  EditList *GetEditList()       { return edl; }
  void SetEditList(EditList *l) { edl=l; }
  const EditList  & GetEditList()  const { return *edl; }
};

typedef AZDList <ListItemBase>      ListItemBaseDList;
typedef AZDList <ListItem>          ListItemDList;
typedef AZDList <ListItemSelWindow> ListItemSelWindowDList;
typedef AZDList <ListItemEditList>  ListItemEditListDList;

class _WindowListe_: public Window
{
protected:
  int               listCount;
  int               topItem;
  int               listVisible;
  int               selectItem;
  int               style;
  ListItemBaseDList liste;

  virtual void PaintItem(const ListItemBase &,bool selected);
         ListItemBase * GetItem();
  const ListItemBase * GetItem() const;
        ListItemBaseDList & GetList()       { return liste; }
  const ListItemBaseDList & GetList() const { return liste; }

public:
  _WindowListe_(int _x,int _y,int _w,int _h, int style);
  void AddItem(ListItemBase *);
  int  GetItemCount()     const { return listCount;  }
  void SelectItem(int count);
  void RemoveItem(int count);
  void Paint();
  int GetSelectedItem()  const { return selectItem; }
  void DeleteAll() { liste.DeleteAll(); listCount=0; selectItem=-1; }
  void * GetItemByID(int id);
  const void * GetItemByID(int id) const;
};

template <class T> class WindowListe : public _WindowListe_
{
public:
  WindowListe(int _x,int _y,int _w,int _h, int style)
    :_WindowListe_(_x,_y,_w,_h,style) {}
  void AddItem(T *it) { _WindowListe_::AddItem(it); }
  const T * GetItem() const
                       { return (const T*)(_WindowListe_::GetItem()); }
        AZDList<T> & GetList()
                       { return (AZDList<T>&)_WindowListe_::GetList(); }
  const AZDList<T> & GetList() const
                       { return (const AZDList<T> &)_WindowListe_::GetList(); }
  const T * GetItemByID(int id) const { return (const T*)_WindowListe_::GetItemByID(id); }
};

class PartitionWindow : public Window
{
  WindowListe <ListItemPartEntry> partList;
  PartEntryList & peList;

public:
  PartitionWindow(PartEntryList &peList);
  void Paint();
  void Up();
  void Down();
  void Top();
  void Bottom();
  const PartEntry & GetSelectedItem();
  const PartEntryList & GetPartEntryList() const { return peList; }
};

class HelpWindow : public Window
{
  AZString text;
public:
  HelpWindow();
  void Paint();
  void SetText(const char *t);
};

class EditString : public AZString
{
  bool     isConst;
  int      cursor;
  int      editLen; // die eigentliche textlaenge
  bool     moveCursor;
  AZString filter_str;
public:                           // soll sich er cursor bewegen
                                             //bitte nur diese zeichen
  EditString(const char *t,bool c,bool _move,const char *filter="");
  bool IsConst() const    { return isConst; }
  // sagt ob sich der cursor bewegen soll
  // bsp: flag argument [x] : dort soll er sich nicht bewegen
  bool MoveCursor() const { return moveCursor; }
  int  GetCursor() const  { return cursor; }
  void SetCursor(int cr)  { cursor=max(min(cr,Len()),0); }
  // ret=false char c wurde nicht akzeptiert
  virtual bool SetChar(char c);
  int EditLen() const     { return editLen; }
  void SetEditLen(int l)  { editLen=l; }
};

class EditIntString : public EditString
{
public:
  EditIntString(const char *t,bool c)
    :EditString(t,c,true) {}
  bool SetChar(char c);
};

class EditFlagString : public EditString
{
public:
  EditFlagString(const char *t,bool c)
    :EditString(t,c,false) {}
  bool SetChar(char c);
};

class EditList : public AZDList <EditString>
{
private:
  void AddVFlg(const char *); // die gibts nicht!!

public:
  EditList() {}
  void AddCStr(const char *t)
  {
    EditString *e=new EditString(t,true,true);
    if (e) AddTail(e);
  }
  void AddVStr(const char *t,const char *filter_str="");
  void AddVInt(const char *t)
  { EditString *e=new EditIntString(t,false); if (e) AddTail(e); }
  void AddVFlg(bool set)
  { EditString *e=new EditFlagString(set?"x":" ",false); if (e) AddTail(e); }
  const EditString & operator[](int index) const;
};

class WindowEditListe : public WindowListe <ListItemEditList>
{
  typedef WindowListe <ListItemEditList> inherited;

  int h_selected;
  void Up();
  void Down();
  void Top();
  void Bottom();
  void Left();
  void Right();
  void Tab();
  void ShiftTab();
  void Enter();
  void Entf();

public:
  WindowEditListe(int _x,int _y,int _w,int _h, int style)
    :inherited(_x,_y,_w,_h,style),h_selected(0) {}
  void PaintItem(const ListItemBase &,bool selected);
  int  GetHSelectedItem() const { return h_selected; }
  int  GetHItemCount();
  void HSelectItem(int i) { h_selected=i; }
  bool InsertChar(char c);
  EditString * GetSelectedEditString();
  void SetCommand(char c);
//  void DeleteAll() { inherited::DeleteAll(); h_selected=0; }
};

// nun das umgekehrte aus einem 'entrie' wieder einen 'pe' machen
const PartEntry *FindEntry(PartEntryList &peList,
                           const par_partitionentry &en);

class SelWindow : public Window
{
public:
  WindowListe <ListItemSelWindow> partEditList;

  SelWindow(PartEntryList &peList,const ImportSetting & im);
  void Paint();
  void Insert(const PartEntry &,EditList *edl=NULL);
  void Delete();

  bool ListNotEmpty() const { return (partEditList.GetItemCount()>0)?true:false; }
  const ListItemSelWindow & operator [](int index) const;

};

class PartEditWindow: public WindowListe <ListItem>
{
  typedef WindowListe <ListItem> inherited;

  AZString hiding_map_print;
  AZString hiding_map; // mit ' ' und lower_case

public:
  WindowEditListe partEditList; // edl-kopie von SelWindow

  PartEditWindow();  // selektierte partitionen (b) liste aller partitionen
  void Init(SelWindow &,const PartEntryList & peList);
  void Paint();
  bool InsertChar(char c);
  void SetCommand(char c) { partEditList.SetCommand(c); }


  static EditList * CreateEditList(int nr, // des eintrages in der liste
                                   const char entryname[32],
                                   byte  parameter,
                                   const char hiding_map[10],
                                   const AZString & hiding_map_possible);
};

// warum 2x WindowListe?
// einmal damits keinen rahmen hat
// und einmal weil es wirklich eine liste ist
class BootEditWindow: public WindowListe <ListItemEditList>
{
  typedef WindowListe <ListItemEditList> inherited;

  WindowEditListe partList;
public:
  BootEditWindow(const ImportSetting & im);
  void Paint();
  bool InsertChar(char c);

  void Init();
  void SetCommand(char c) { partList.SetCommand(c); }
  const ListItemEditList *GetItemByID(int id) const
     { return partList.GetItemByID(id); }
};

class BootSelWindow:public WindowListe <ListItem>
{
  typedef WindowListe <ListItem> inherited;

public:
  BootSelWindow();
  void Init(PartEditWindow *);
};

class InstallHinweis:public WindowListe <ListItem>
{
  typedef WindowListe <ListItem> inherited;

public:
  InstallHinweis();
  void Paint();
};

class InstallEdit:public WindowEditListe
{

public:
  bool writeToMBR;
  bool writeToDisk;
public:
  InstallEdit();
  void Paint();
  bool InsertChar(char c);
  void SetCommand(char c);
};

class TitelZeile : public CWindow
{
  typedef CWindow inherited;
public:
  TitelZeile():CWindow(0,0,80,1,WIN_NO_FRAME)
  {
    SetColor(COL_TITLE);
  }
  void Paint();
};

class FussZeile : public CWindow
{
  typedef CWindow inherited;
public:
  FussZeile():CWindow(0,24,80,1,WIN_NO_FRAME)
  {
    SetColor(COL_TITLE);
  }
  void Paint();
};

