// *********************************************************************
// *                                                                   *
// * (c) 1997-2004 Andreas Ziermann LOET.DE                            *
// *                                                                   *
// * These files are for reference only. You may not use these files   *
// * for commercial use without permit.                                *
// *                                                                   *
// * b@loet.de                                                         *
// *                                                                   *
// * 1.00 2004-07-04                                                   *
// *      first released                                               *
// *                                                                   *
// *********************************************************************
#include <baselib/all.h>

#include "biosdefs.h"
#include "partentry.h"
//#include "debug.h"
#include "version.h"
#include "myargs.h"

// partitionentries fuer dos/win/winnt laufwerke
byte doslw[]={0x01,0x04,0x06,0x07,0x0b,0x0c,0x0e,
              0x11,0x14,0x16,0x17,0x1b,0x1c,0x1e};

#define IDS_HARDDISK "Harddisk"


HardDisk::HardDisk(byte _nr)
:nr((byte)(_nr|0x80))
{}

AZString HardDisk::GetText() const
{
  char buffer[256];
  const char *s=IDS_HARDDISK;
  sprintf(buffer,"%s %d",s,nr&0x7f);
  return AZString(buffer);
}

byte HardDisk::AddPartitionsFirstDos(PartEntryList &peList,char &lw) const
{
  PartitionEntry peMBR;
  memset(&peMBR,0,sizeof(PartitionEntry));
  peMBR.start_zyl=1; // sektor 1 on zyl 0 head 0

  // suche partitionstabelle
  PartitionTableHeap t;
  bool ret=BIOSGetPartitionTable(nr,&t());
  if (!ret)
  {
    return 5;
  }
  // suche die erste partition
  byte firstDos=5;
  for (byte i=0;i<4;i++)
  {
    byte id=t().entry[i].part_id;
    bool active=t().entry[i].active==0x80?true:false;
    if (memchr(doslw,id,sizeof(doslw)))
    {
      firstDos=min(firstDos,i);
      if (active)
      {
        firstDos=i;
        break;
      }
    }
  }
  if (firstDos<5)
  {
    AddPartition(peList,PRIM,lw,t().entry[firstDos],peMBR,firstDos,0l);
  }
  return firstDos;
}


void HardDisk::AddPartitionsPrimDos(PartEntryList &peList,char &lw, byte index) const
{
  PartitionEntry peMBR;
  memset(&peMBR,0,sizeof(PartitionEntry));
  peMBR.start_zyl=1; // sektor 1 on zyl 0 head 0

  // suche partitionstabelle
  PartitionTableHeap t;
  bool ret=BIOSGetPartitionTable(nr,&t());
  if (!ret)
  {
    return;
  }
  for (byte i=0;i<4;i++)
  {
    byte id=t().entry[i].part_id;
    if (id!=0 && id!=0x5 && id!=0x0f && i!=index)
      AddPartition(peList,PRIM,lw,t().entry[i],peMBR,i,0l);
  }
}

void HardDisk::AddPartitionsExt(PartEntryList &peList,char &lw) const
{
  // suche partitionstabelle
  PartitionTableHeap t;
  bool ret=BIOSGetPartitionTable(nr,&t());
  if (!ret)
  {
    return;
  }
  for (int i=0;i<4;i++)
  {
    byte id=t().entry[i].part_id;
    if (id==0x05 || id==0x0f )
    {
      dword lba_offset=t().entry[i].rel_start;
      t().entry[i].rel_start=0;
      AddPartitionExt(peList,lw,t().entry[i],lba_offset,lba_offset);
    }
  }
}

void HardDisk::AddPartitionExt(PartEntryList &peList,char &lw,
                               const PartitionEntry &pe,
                               dword lba_offset,dword rel_start) const

{
  PartitionTableHeap t;
  bool ret=BIOSGetPartitionTableExt(nr,rel_start,&t());
  if (!ret)
    return;
  for (byte i=0;i<4;i++)
  {
    byte id=t().entry[i].part_id;
    if (id==0)
      continue;
    if (id==0x05 || id==0x0f )
      AddPartitionExt(peList,lw,t().entry[i],lba_offset,
                      lba_offset+t().entry[i].rel_start);
    else
      AddPartition(peList,LOG,lw,t().entry[i],pe,i,
      pe.rel_start+lba_offset);
  }
}


void HardDisk::AddPartition(PartEntryList &peList,PrimLogExt prim,
                            char &lw,
                            const PartitionEntry &pe,
                            const PartitionEntry &peWoher,
                            byte   index,dword lba_offset) const
{
  AZString type;
  AZString volName;
  AZString lauf("  ");
  char *s=(char *)lauf.Text();
  s[0]=lw;
  s[1]=':';
  char dosName=0;
  PAR_TYPE par_type=UNKNOWN;

  assert(pe.part_id!=0);
  assert(pe.part_id!=5);
  assert(pe.part_id!=0xf);

  Sector bs;
  bool isLoad=BIOSLoadSectorLBA(nr,lba_offset+pe.rel_start,&bs);

  switch ( pe.part_id )
  {
    case 0:
    case 0xf:
    case 5:
    case 0x1:
      type="FAT12                    ";
      goto volname;
    case 0x11:
      type="FAT12         (versteckt)";
      goto volname;
    case 0x4:
      type="FAT16<32M                ";
      goto volname;
    case 0x14:
      type="FAT16<32M     (versteckt)";
      goto volname;
    case 6:
      type="FAT16>32M                ";
      goto volname;
    case 0x16:
      type="FAT16>32M     (versteckt)";
    volname:
      dosName=lw++;
      par_type=UNKNOWNDOS;

      //if (BIOSGetSector(nr,pe.start_head,
      //                     pe.start_zyl,&bs) )
      if (isLoad)
      {
        BootSectorFAT &b=bs.bootsectorfat;
        //char *f1=b.fsid;
        //char *f2="FAT";
        //if (strncmp(f1,f2,3)!=0)
        //  return;
        // detect OS

        // boot lw=1fd
        if (memcmp(b.oemid,"IBM  3.3",8)==0)
        {
          // bootlaufwerk steht im bootsektor nicht an adr 36
          // sondern 1fd
          memcpy((void *)(type.Text()+5)," DOS3.3",7);
          par_type=DRDOS60;
        }
        if (memcmp(b.oemid,"MSWIN4.1",8)==0)
        {
          memcpy((void *)(type.Text()+5)," Win9x ",7);
          par_type=WIN9X;
        }
        // win95 ohne A
        if (memcmp(b.oemid,"MSWIN4.0",8)==0)
        {
          memcpy((void *)(type.Text()+5)," Win95 ",7);
          par_type=WIN9X;
        }
        if (memcmp(b.oemid,"MSDOS4.0",8)==0)
        {
          memcpy((void *)(type.Text()+5)," DOS4.x",7);
          par_type=MSDOS401;
        }
        if (memcmp(b.oemid,"NWDOS7.0",8)==0)
        {
          memcpy((void *)(type.Text()+5)," NWDOS7",7);
          par_type=NWDOS7;
        }
        if (memcmp(b.oemid,"OPENDOS7",8)==0)
        {
          memcpy((void *)(type.Text()),"OpenDos7 ",9);
          par_type=OPENDOS7;
        }
        if (memcmp(b.oemid,"IBM 20.0",8)==0)
        {
          memcpy((void *)(type.Text()+5)," OS/2  ",7);
          par_type=IBMDOS;
        }

        volName=AZString(b.vollabel,11);
        // if (b.signatur==0xaa55 && (volName=="           " || b.vollabel[0]==0))
        {
          char buf[32];
          sprintf(buf,"%s\\*.*",lauf.Text());
          // bios search for vollabel
          // AZString volLabel;
          BIOSFat16_Fat12_FindVolLabel(volName,nr,b,lba_offset+pe.rel_start);
          //struct ffblk ffblk;
          //if (findfirst(buf,&ffblk,FA_LABEL)==0)
          //  volName=AZString(ffblk.ff_name,8);    // ff_name[13]=8+'.'+3+'\0'
          //  volName+=AZString(ffblk.ff_name+9,3);
        }
      }
      volName=lauf+AZString("  ")+volName;
      break;
    case 0x0b:
      type="FAT32                    ";
      goto volname32;
    case 0x1b:
      type="FAT32         (versteckt)";
    volname32:
      dosName=lw++;
      par_type=FAT32;

      // bool BIOSLoadSectorLBA(byte hdnr,dword lbaADR,Sector *b);
      if (isLoad)
      {
        BootSectorFAT32 &b=bs.bootsectorfat32;
        volName=AZString(b.vollabel,11);
        if (memcmp(b.oemid,"MSWIN4.1",8)==0)
          memcpy((void *)(type.Text()+5)," Win9x ",7);

      }
      volName=lauf+AZString("  ")+volName;
      break;
    case 0x0c:
      type="FAT32 LBA                ";
      goto volname32;
    case 0x1c:
      type="FAT32 LBA     (versteckt)";
      goto volname32;
    case 0xe:
      type="WIN9x LBA                ";
      goto volname;
    case 0x1e:
      type="WIN9x LBA     (versteckt)";
      goto volname;
    case 0x82: // linux swap
      type="Linux swap               ";
      volName="               ";
      par_type=LINUX;
      break;
    case 0x83:
      type="Linux native             ";
      volName="    Linux      ";
      par_type=LINUX;
      break;
    case 0x0a:
      type="                         ";
      volName="OS/2 Bootmanager";
      par_type=OS2BOOTMAN;
      break;
    case 0x07:
      type="HPFS/NTFS                ";
      goto volnameHPFS;
    case 0x17:
      type="HPFS/NTFS     (versteckt)";
    volnameHPFS:
    {
      dosName=lw++;
      // OEMID "OS2 20.0"
      BootSectorOS2 s=bs.bootsectoros2;
      // wenn part 7 und windows 2000, dann steht dort ntfs
      if (strncmp(s.oemid,"NTFS     ",8)==0)
      {
        memcpy((void *)(type.Text()),"NTFS     ",9);
        volName=lauf+AZString("  ")+volName;
        par_type=NTFS;
      }
      else
      {
        volName=AZString(s.vollabel,11);
        par_type=OS2HPFS;
      }
      break;
    }
    default:
      type="                         ";
      volName="               ";
      sprintf((char *)type.Text(),
              "unknown Partition 0x%02x   ",pe.part_id);
      assert(strlen(type.Text())==25);
      par_type=UNKNOWN;
  }
  PartEntry *en=new PartEntry(nr, par_type, type, volName.Text(),
                              prim, dosName, pe, peWoher,index,
                              lba_offset+pe.rel_start,
                              lba_offset,(bs.bootsector.signatur==0xaa55)?true:false);
  if (en)
      peList.AddTail(en);
}

//************************************************************                                         *
//*                                                          *
//* FillPeList(PartEntryList &)                              *
//*                                                          *
//************************************************************
void FillPeList(PartEntryList &peList)
{
  unsigned hdcount=BIOSHarddiskCount();
  char c='C';

  byte *index_tbl=new byte[hdcount];
  for (unsigned i_=0;i_<hdcount;i_++)
  {
    HardDisk hd((byte)i_);
    byte ret=hd.AddPartitionsFirstDos(peList,c);
    if (index_tbl)
      index_tbl[i_]=ret;
  }

  for (unsigned i__=0;i__<hdcount;i__++)
  {
    HardDisk hd((byte)i__);
    hd.AddPartitionsExt(peList,c);
  }
  for (unsigned i=0;i<hdcount;i++)
  {
    HardDisk hd((byte)i);

    byte index=5;
    if (index_tbl)
      index=index_tbl[i];

    hd.AddPartitionsPrimDos(peList,c,index);
  }

  // nun noch die floppy hinzufuegen
  PartitionEntry pe_floppy;
  memset(&pe_floppy,0,sizeof(PartitionEntry));
  pe_floppy.start_zyl=1;

  PartEntry *en=new PartEntry(0,FLOPPYBOOTSEC,"von Diskette booten      ",
                                "A:  FloppyDisk",EXT,0,pe_floppy,pe_floppy,0,0,0,true);
  if (en)
      peList.AddTail(en);

}


PartEntry::PartEntry(byte _hdnr,PAR_TYPE type,const char *_parTyp,
                     const char *_volName, PrimLogExt _prim,
                     char  _dosName,
                     const PartitionEntry & _pe,
                     const PartitionEntry & _peWoher,
                     byte   index,
                     dword lbaBootsec,dword lbaWoher,
                     bool _signatur)
:hdnr(_hdnr)
,par_type(type)
,parTyp(_parTyp)
,volName(_volName)
,prim(_prim)
,dosName(_dosName)
,indexInTable(index)
,peBootSec(_pe)
,lbaADRpeBootSec(lbaBootsec)
,peWoher(_peWoher)
,lbaADRpeWoher(lbaWoher)
,signatur(_signatur)
{
}

bool PartEntry::IsBootAble() const
{
  if (!signatur)
    return false;
  if (par_type==IBMDOS)
  {
    if (prim==PRIM && hdnr==0x80)
      return true;
    else
      return false;
  }

  if (prim==LOG)
  {
    // wwas koennen wir nicht von einer logischen partition booten
    switch (par_type)
    {
      case UNKNOWNDOS:
      case WIN9X:
      case FAT32:
      case MSDOS401:
      case OS2BOOTMAN:
      case OS2HPFS:
        return false;
      default:
        break;
      // DRDOS60,NWDOS7,OPENDOS7 funktionieren, wenn mal alle prim.Part.
      // versteckt
      // LINUX und jedes andere vernuenftige OS solle auch gehen
    }
  }
  return true;
}

AZString PartEntry::GetSize() const
{
  dword seccount=peBootSec.sec_count;
  if (!seccount)
    return AZString("");

  char zahl[24]="";
  zahl[16]=0; // hinten komtm noch ein " MB" dran
  char *zptr=zahl+15;
  dword sec=seccount/2048; // *512/1024/1024
  // sec/=1024; // in MByte
  byte rest=0;
  int dot=0;
  do
  {
    if (dot==3)
    {
      *zptr--='.';
      dot=0;
    }
    dot++;

    rest=(byte)(sec%10);
    *zptr--=(byte)(0x30+rest);

    sec/=10;
  } while(sec);
  zptr++;
  strcat(zptr," MB");
  return AZString(zptr);
}

AZString PartEntry::GetTextLong() const
{
  dword seccount=peBootSec.sec_count;

  char buffer[256];
  char *p1="primr ";
  char *p2="logisch";
  char *p3="       ";
  AZString _volName(volName);
  _volName+="                 ";
  char zahl[18]="";
  zahl[16]=0;
  char *zptr=zahl+15;
  dword sec=seccount/2048; // *512/1024/1024
  // sec/=1024; // in MByte
  byte rest=0;
  int dot=0;
  do
  {
    if (dot==3)
    {
      *zptr--='.';
      dot=0;
    }
    dot++;

    rest=(byte)(sec%10);
    *zptr--=(byte)(0x30+rest);

    sec/=10;
  } while(sec);
  zptr++;

  char *c=(char *)_volName.Text();
  c[17]=0;
  sprintf(buffer,"%s:%d   %s%s  %s%12s %s %s",hdnr&0x80?"hd":"fd",
          hdnr&0x7f,
          _volName.Text(),parTyp.Text(),prim!=LOG?(prim==EXT?p3:p1):p2,
          seccount?zptr:"",seccount?"MB":"  ",IsBootAble()?"  ":"??");
  return AZString(buffer);
}

AZString PartEntry::GetTextShort() const
{
  dword seccount=peBootSec.sec_count;

  char buffer[100];

  AZString _volName(volName);
  _volName+="                 ";
  char *c=(char *)_volName.Text();
  c[16]=0;

  char zahl[17];
  zahl[16]=0;
  char *zptr=zahl+15;

  dword sec=seccount/2048; // *512/1024/1024
  byte rest=0;
  int dot=0;
  do
  {
    if (dot==3)
    {
      *zptr--='.';
      dot=0;
    }
    dot++;

    rest=(byte)(sec%10);
    *zptr--=(byte)(0x30+rest);

    sec/=10;
  } while(sec);
  zptr++;

  const char *v=_volName.Text();
  const char *p=parTyp.Text();
  const char *mb="MB";
  const char *mbl="  ";
  mb=seccount?mb:mbl;

  memset(buffer,20,90);
  buffer[90]=0;

  sprintf(buffer,"%s%s%9s %s %s",v,p,seccount?zptr:"",mb,
          IsBootAble()?"  ":"??");
  return AZString(buffer);
}
