// *********************************************************************
// *                                                                   *
// * (c) 1997-2004 Andreas Ziermann LOET.DE                            *
// *                                                                   *
// * These files are for reference only. You may not use these files   *
// * for commercial purpose use without permit.                        *
// *                                                                   *
// * b@loet.de                                                         *
// *                                                                   *
// * 1.00 2004-07-04                                                   *
// *      first released                                               *
// *                                                                   *
// *********************************************************************

#include <baselib/all.h>

#include <conio.h>
#include <dos.h>

#include "biosdefs.h"
#include "asmdefs.h"
#include "impsett.h"
#include "mouse.h"
#include "partentry.h"
#include "mywin.h"

void Window::gotoxy(int _x,int _y)
{
  ::gotoxy(x+2+_x,_y+3+y);
}

Window::Window(int _x,int _y,int _w,int _h)
:x(_x),y(_y),w(_w),h(_h) {}

void Window::Paint()
{
  int myy=y+2;
  int myx=x+1;

  ::gotoxy(myx,myy++);
  textattr(COL_NORMAL);
  putch(TOP_LEFT);
  for(int i_=0;i_<w;i_++) putch(TOP);
  putch(TOP_RIGHT);

  for (int i=0;i<h;i++,myy++)
  {
    ::gotoxy(myx,myy);
    putch(LEFT);
    for(int i=0;i<w;i++) putch(SPACE);
    putch(RIGHT);
  }
  ::gotoxy(myx,myy);
  putch(BOTTOM_LEFT);
  for(int i__=0;i__<w;i__++) putch(BOTTOM);
  putch(BOTTOM_RIGHT);
}


//************************************************************************
//
// CWindow
//
//************************************************************************

void CWindow::DrawText(int relx,int rely,const char *src,byte col)
{
  if (!buffer_char || !src || (col && !buffer_attrib))
    return;
  assert(relx<w);
  assert(rely<h);
  if (relx>=w)
    return;
  if (rely>=h)
    return;
  int _w=w;
  int _relx=relx;
  if (!(flags&WIN_NO_FRAME))
  {
    _relx++;
    rely++;
    _w+=2;
  }
  byte *dest       =buffer_char+_relx+(rely*_w);
  if (col)
  {
    byte *dest_attrib=buffer_attrib+_relx+(rely*_w);
    memset(dest_attrib,col,min((int)(w-relx),(int)strlen(src)) );
  }
  memcpy(dest,src,min((int)(w-relx),(int)strlen(src)) );
}

CWindow::CWindow(int _x,int _y,int _w,int _h,word _flags)
:x(_x),y(_y),w(_w),h(_h),flags(_flags)
,attr(COL_NORMAL),buffer_attrib(NULL)
,buffer_char(NULL)
{
  buffer_size=w*h;
  if (!(flags&WIN_NO_FRAME))
  {
    buffer_size+=2*w+2*h+4;
  }
  buffer_attrib=new byte[buffer_size];
  buffer_char=new byte[buffer_size];
}

void CWindow::Paint()
{
  int add=0;

  if (!(flags&WIN_NO_FRAME) && buffer_char)
  {
    // aha es hat also ein frame
    byte *start=buffer_char;
    *start++=TOP_LEFT;
    memset(start,TOP,w);
    start+=w;
    *start++=TOP_RIGHT;
    for (int i=0;i<h;i++)
    {
      *start++=LEFT;
      start+=w;
      *start++=RIGHT;
    }
    *start++=BOTTOM_LEFT;
    memset(start,BOTTOM,w);
    start+=w;
    *start=BOTTOM_RIGHT;
    add=2;
  }

  #ifdef USE_MOUSE
  bool b_was_visible=mouse_visible();
  if (b_was_visible)
    mouse_off();
  #endif

  byte *buf_attrib=buffer_attrib;
  byte *buf_char  =buffer_char;
  REGPACK r;

  for (int _y=y; _y<y+h+add; _y++)
  {
    for (int _x=x; _x<x+w+add; _x++,buf_attrib++,buf_char++)
    {

      r.r_ax=0x0200; // ah==0
      r.r_bx=0;
      r.r_dx=(_y<<8)|_x; // cursor x=0 y=0
      intr(0x10,&r);

      byte c=buffer_char?*buf_char:' ';
      byte a=buffer_attrib?*buf_attrib:COL_NORMAL;

      r.r_ax=(0x9<<8)|c;
      r.r_bx=a;
      r.r_cx=1;
      intr(0x10,&r);
    }
  }
  #ifdef USE_MOUSE
  if (b_was_visible)
    mouse_on();
  #endif
}


void CWindow::Clear()
{
  if (buffer_attrib)
    memset(buffer_attrib,attr,buffer_size);
  if (buffer_char)
    memset(buffer_char,' ',buffer_size);
}

void CWindow::SetColor(byte _attr)
{
  attr=_attr;
}

void CWindow::Center()
{
  int add=0;

  if (!(flags&WIN_NO_FRAME))
  {
    add=2;
  }
  x=(80-w-add)/2;
  y=(25-h-add)/2;
}

word CWindow::Execute()
{
  do
  {
    word ch=getch();
    if (ch==0x0d || ch==0x1b)
      return ch;
  }
  while(1);
}

